#include "flashprog.h"
#include "interrupts.h"
#include "twi.h"
#include "systick.h"
#include "nxt_avr.h"

#include <sil.h> // ASP
#include "target_coproc.h" // ASP

int
flash_write_page(int start_page, U32 *page, int page_num)
{
  int i, istate;
  
  if (page_num + start_page > 1023) return 0;
  
  at91sam7s_disable_int(INTNO_BITPAT(INTNO_LOWPIRQ));
  //systick_suspend();
   	
  sil_dly_nse(100000U); // ASP
  //systick_wait_ms(1);
 
  nxt_avr_1kHz_update();
 
  while (twi_busy());
  
  sil_dly_nse(100000U); // ASP
  //systick_wait_ms(1);
  
  istate = interrupts_get_and_disable();

  while (!(FLASH_STATUS_REG & 0x1));

  for (i = 0; i < 64; i++)
    FLASH_BASE[(page_num*64)+i] = page[i];

  FLASH_CMD_REG = (0x5A000001 + (((page_num + start_page) & 0x000003FF) << 8));

  while (!(FLASH_STATUS_REG & 0x1));
  
  if (istate) interrupts_enable();
  
  at91sam7s_enable_int(INTNO_BITPAT(INTNO_LOWPIRQ));
  //systick_resume();
  
  return 1;
}

void flash_set_mode(U32 fmcn) {
  FLASH_MODE_REG = ((fmcn << 16) | (1 << 8));
}

void
flash_erase_range(U32 addr, U32 nBytes)
{
  int i = 0;
  int istate = interrupts_get_and_disable();

  while (nBytes--) {
    i++;
  }
  if (istate)
    interrupts_enable();
}

void
flash_write(U32 addr, void *buffer, U32 nBytes)
{
  int i = 0;
  int istate = interrupts_get_and_disable();

  while (nBytes--) {
    i++;
  }
  if (istate)
    interrupts_enable();
}
