
#include <string.h>

#include "ecrobot_base.h"
#include "ecrobot_private.h"
#include "ecrobot_interface.h"

#include "arm.h"
#include <t_stddef.h>
#include <sil.h>
#include "kernel_cfg.h"
#include "kernel_impl.h"
//#include "time_event.h"
#include "target_main.h"
#include "systick.h"

#include "nxt_config.h"

/* If there was no operation on the NXT within SLEEP_TIME
 * after starting the system, turn the NXT into sleep (shut down)
 * to avoid battery discharge.
 */
#define SLEEP_TIME (1000*60*10) /* 10min */

void
nxt_main_task(intptr_t exinf)
{
	uint32_t st;
	uint32_t last_act_time = 0;
	uint32_t flash_req_cnt = 0;

	show_splash_screen();
	show_main_screen();
	display_status_bar(1); /* clear status bar */
	add_status_info(execution_mode());
	display_status_bar(0); /* update status bar */

	while(1)
	{
		ecrobot_init_bt_slave(BT_PASS_KEY);
		/* check the buttons every 10msec */
		get_tim(&st);
		if (st >= last_act_time + 10) 
		{
			last_act_time = st;
			ecrobot_poll_nxtstate();
			display_status_bar(0);

			/* 
			 * executed in FLASH: setup for the application flash
			 * executed in SRAM:  no effect
			 */
			if ((ecrobot_get_button_state() == (ENTER_PRESSED | STOP_PRESSED)) &&
			    (execution_mode() == EXECUTED_FROM_FLASH))
			{
				flash_req_cnt++;
				/* keep pusing ENTER + STOP buttons more than 1000msec */
				if (flash_req_cnt >= 100)
				{
					/* set flash request and shut down the NXT
				 	 * at the next start, NXT BIOS will be executed.
				 	 */
					ecrobot_device_terminate();
					set_flash_request();
					display_clear(1);
					sil_dly_nse(50000U);
					nxt_lcd_power_down(); /* reset LCD hardware */
					sil_dly_nse(50000U);
					while(1)
					{
						nxt_avr_power_down();
					}
				}
			}
			else
			{
				flash_req_cnt = 0;
				if ((ecrobot_get_button_state() == EXIT_PRESSED) || (st > SLEEP_TIME))
				{
					/* shut down the NXT */
					ecrobot_device_terminate();
					display_clear(1);
					sil_dly_nse(50000U);
					nxt_lcd_power_down(); /* reset LCD hardware */
					sil_dly_nse(50000U);
					while(1)
					{
						nxt_avr_power_down();
					}
				}
				else if (ecrobot_get_button_state() == RUN_PRESSED)
				{
					display_clear(1);
					sil_dly_nse(50000U);

					sta_cyc(CHECK_BUTTON);
					start_task_cyc();
					
					ext_tsk();
					/* never reached here */
				}
			}
		}
    }

}


