/*****************************************************************************
 *
 * COPYRIGHT 2008 Takashi Chikamasa <takashic@cybernet.co.jp>
 *
 * <About leJOS NXJ>
 *  leJOS NXJ is a full firmware replacement of LEGO Mindstorms NXT and 
 *  designed for Java programming environment for the NXT 
 *  ( For more detailed information, please see: http://lejos.sourceforge.net/ )
 *  In the leJOS NXJ distribution, C source files for NXT platform layer is also
 *  included besides with the Java VM. The platform C source code is well
 *  structured, comprehensive, and achieved higher performance than the LEGO's
 *  one. Therefore, leJOS NXJ (platform) is also the best GCC based C/C++  
 *  development platform for NXT.
 *
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is TinyVM code, first released March 6, 2000,
 *  later released as leJOS on September 23, 2000.
 *
 *  The Initial Developer of the Original Code is Jose H. Solorzano.
 *
 *  Contributor(s): see leJOS NXJ ACKNOWLEDGEMENTS .
 *
 *
 * <About TOPPERS ASP>
 *  TOPPERS ASP is an open source kernel and developed by TOPPERS project.
 *  TOPPERS(Toyohashi OPen Platform for Embedded Real-time Systems) has been managed 
 *  by a Non Profit Organization founded in Sep. 2003 and has been led by Professor
 *  Hiroaki Takada of Nagoya University in Japan. 
 *
 *  TOPPERS ASP program is covered by the TOPPERS License as published
 *  by the TOPPERS PROJECT (http://www.toppers.jp/en/index.html).
 *
 *****************************************************************************/

#include <string.h>

#include <stdio.h>

#include "ecrobot_base.h"
#include "ecrobot_private.h"
#include "ecrobot_interface.h"

#include <t_stddef.h>
#include "nxt_config.h"
#include <sil.h>
#include "kernel_impl.h"
#include "interrupt.h"
#include "tMainNXT.h"

extern void target_stop(void);

/* C++ constructor/destructor table */
extern void (*__ctor_begin[])(void);
extern void (*__ctor_end)(void);
extern void (*__dtor_begin[])(void);
extern void (*__dtor_end)(void);

/* used for getting the address of a constant data */
const UINT exec_addr[1] = {0};

static CHAR status_bar[MAX_STATUS_BAR_LENGTH];
static SINT status_bar_position = 0;

uint_t reset_flg = 0;


/*
 * execute C++ constructors
 */
void
cpp_constructor(void)
{
    SINT i;

    /* call all global object constructors */
    for (i=0;__ctor_begin[i] != __ctor_end;i++)
      (*__ctor_begin[i])();
}

/*
 * execute C++ destructors
 */
void
cpp_destructor(void)
{
    SINT i;

    /* call all global object destructors */
    for (i=0;__dtor_begin[i] != __dtor_end;i++)
      (*__dtor_begin[i])();
}

/*
 * return where the program is executed
 */
SINT
execution_mode(void)
{
	if (exec_addr < (volatile UINT *)0x200000)
	{
		return EXECUTED_FROM_FLASH;
	}
	return EXECUTED_FROM_SRAM;
}

#include "kernel_cfg.h"

volatile bool_t screen_endflag = false;

void
alarm_screen(intptr_t exinf)
{
	screen_endflag = true;
}

/*
 * display application splash screen which is designed in
 * ecrobot_splash.bmp file
 */
void
show_splash_screen(void)
{
	display_clear(0);
	display_goto_xy(0, 0);
	display_string("TOPPERS/ASP");
	display_goto_xy(0, 1);
	display_string("Kernel");
	display_goto_xy(0, 2);
	display_string("Release 1.4.0");
	display_goto_xy(0, 3);
	display_string("Copyright (C)");
	display_goto_xy(0, 4);
	display_string("2009 by ERTL");
	display_goto_xy(0, 5);
	display_string("Nagoya Univ.");
	display_update();

	sta_alm(ALM_SCREEN, 1000);
	while(!screen_endflag);
	//systick_wait_ms(1000);
}

/*
 * display application main screen
 */
void
show_main_screen(void)
{
	display_clear(0);
	display_goto_xy(0, 5);
	display_string("================");
	display_goto_xy(0, 6);
	display_string("<STP  ENTR  RUN>");
	display_goto_xy(0, 7);
	display_string("      EXIT      ");
	display_update();
}

/*
 * display status bar in the main screen
 * 
 * clear: 1(clear status bar), 0(not clear status bar)
 */
void
display_status_bar(SINT clear)
{
	if (clear)
	{
		memset(status_bar, ' ', MAX_STATUS_BAR_LENGTH);
		status_bar_position = 0;
	}

	display_goto_xy(0, 3);
   	display_string("BATT:");
   	display_unsigned(ecrobot_get_battery_voltage()/100, 0);
	display_goto_xy(0, 4);
	display_string(status_bar);
	display_update();
}

void
show_bd_addr(U8 *bd_addr)
{
	SINT i;

	{
		display_goto_xy(0, 0);
		display_string("BD_ADDR:");
		display_goto_xy(0, 1);
		for (i = 0; i < 7; i++)
		{
			display_hex(bd_addr[i], 2);
		}
		display_update();
	}
}

void
show_bd_addr_err(void)
{
	{
		display_goto_xy(0, 0);
		display_string("BD_ADDR: FAILED");
		display_goto_xy(0, 1);
		display_string("RESTART NXT.");
		display_update();
	}
}

/*
 * add information to the status bar
 */
void
add_status_info(SINT status)
{
	CHAR *str;
	
	{
		switch(status)
		{
			case EXECUTED_FROM_FLASH:
				str = "[F]";
				break;
			case EXECUTED_FROM_SRAM:
				str = "[R]";
				break;
/* 			case BT_CONNECTED:
 * 				str = "[BT]";
 * 				break;
 */
			default:
				return;
		}

		if ((status_bar_position + strlen(str)) <= MAX_STATUS_BAR_LENGTH)
		{
			strcpy(&status_bar[status_bar_position], (const CHAR *)str);
			status_bar_position += strlen(str);
		}
	}
}


/*
 * read buttons status on the NXT. This function is assumed to be executed
 * every 1msec
 */
void
check_NXT_buttons(intptr_t exinf)
{
	ecrobot_poll_nxtstate();
	
	if (ecrobot_get_button_state() == STOP_PRESSED) {
//		cpp_destructor();
		ecrobot_device_terminate();
		display_clear(0);
		display_force_update();
		sil_dly_nse(10000000U);
		nxt_lcd_power_down(); /* reset LCD hardware */
		sil_dly_nse(10000000U);
		
		/* Zbg */
		target_stop();
		
	} else if (ecrobot_get_button_state() == EXIT_PRESSED) {
		ext_ker(); /* I & d؂ */
	}
}

/* foCX̏ */
void
nxt_initialize(intptr_t exinf)
{

	nxt_device_init();
	ecrobot_init_nxtstate();

	/* device init should be called prior to running the application */
	nxt_motor_set_count(NXT_PORT_A, 0);
	nxt_motor_set_count(NXT_PORT_B, 0);
	nxt_motor_set_count(NXT_PORT_C, 0);
//	cpp_constructor();

	interrupts_get_and_disable(); // IRQ݋֎~.
}

extern void nxt_avr_term_send(void);

/* I */
void
nxt_terminate(void)
{
	// AVRƒʐM邽IRQ݂
	interrupts_enable();
  
	/* [U`̏I */
//	ecrobot_device_terminate(); // TODO component

	/* fBXvCALCD̓d؂ */
	display_clear(0);
	display_force_update();
	sil_dly_nse(10000000U);
	nxt_lcd_power_down();
	sil_dly_nse(10000000U);
	
	/* AVR̓d؂(AVRɃR}h𑗂) */
	nxt_avr_power_down();
	nxt_avr_term_send();
	sil_dly_nse(5000000U); // R}hSAVRɓ͂܂ő҂Ȃ΂ȂȂ

	// IRQ݂֎~
	interrupts_get_and_disable();
}

/* ^C}nh */
/* void
 * target_timer_handler(void)
 * {
 * 	signal_time();
 * 	systick_isr_C();
 * }
 */

